@@ -29,69 +29,70 @@
 """
 
 
-try:
-    import cStringIO as StringIO
-except ImportError:
-    import StringIO
-
 from xlsxwriter.workbook import Workbook
+from celery import shared_task
+from django.conf import settings
 
 
-def ctxlsx(request):
+
+
+@shared_task
+def ctxlsx(filterdict):
     """Export filtered CT database data to multi-sheet Microsoft XSLX files.
 
-    :param request: Query parameters from the CT filtered page URL.
-    :type request: HTTP get
+    :param filterdict: Query parameters from the CT filtered page URL.
+    :type filterdict: HTTP get
     
     """
 
+    import os, sys, datetime
+    from tempfile import TemporaryFile
+    from django.conf import settings
+    from django.core.files import File
+    from django.shortcuts import redirect
     from remapp.models import General_study_module_attributes
-    from django.http import HttpResponse
+    from remapp.models import Exports
+    from remapp.interface.mod_filters import CTSummaryListFilter
 
-    
+    tsk = Exports.objects.create()
 
-    # Get the database query filters
-    f_institution_name = request.GET.get('general_equipment_module_attributes__institution_name')
-    f_date_after = request.GET.get('date_after')
-    f_date_before = request.GET.get('date_before')
-    f_study_description = request.GET.get('study_description')
-    f_age_min = request.GET.get('patient_age_min')
-    f_age_max = request.GET.get('patient_age_max')
-    f_manufacturer = request.GET.get('general_equipment_module_attributes__manufacturer')
-    f_manufacturer_model_name = request.GET.get('general_equipment_module_attributes__manufacturer_model_name')
-    f_station_name = request.GET.get('general_equipment_module_attributes__station_name')
-    f_accession_number = request.GET.get('accession_number')
-    
-    # Get the data!
-    from remapp.models import General_study_module_attributes
+    tsk.task_id = ctxlsx.request.id
+    tsk.modality = "CT"
+    tsk.export_type = "XLSX export"
+    datestamp = datetime.datetime.now()
+    tsk.export_date = datestamp
+    tsk.progress = 'Query filters imported, task started'
+    tsk.status = 'CURRENT'
+    tsk.save()
+
+    try:
+        tmpxlsx = TemporaryFile()
+        book = Workbook(tmpxlsx, {'default_date_format': 'dd/mm/yyyy',
+                                 'strings_to_numbers':  True})
+        tsk.progress = 'Workbook created'
+        tsk.save()
+    except:
+        messages.error(request, "Unexpected error creating temporary file - please contact an administrator: {0}".format(sys.exc_info()[0]))
+        return redirect('/openrem/export/')
+
+    # Get the data
     e = General_study_module_attributes.objects.filter(modality_type__exact = 'CT')
+    f = CTSummaryListFilter.base_filters
+
+    for filt in f:
+        if filt in filterdict and filterdict[filt]:
+            # One Windows user found filterdict[filt] was a list. See https://bitbucket.org/openrem/openrem/issue/123/
+            if isinstance(filterdict[filt], basestring):
+                filterstring = filterdict[filt]
+            else:
+                filterstring = (filterdict[filt])[0]
+            if filterstring != '':
+                e = e.filter(**{f[filt].name + '__' + f[filt].lookup_type : filterstring})
     
-    if f_institution_name:
-        e = e.filter(general_equipment_module_attributes__institution_name__icontains = f_institution_name)
-    if f_study_description:
-        e = e.filter(study_description__icontains = f_study_description)
-    if f_manufacturer:
-        e = e.filter(general_equipment_module_attributes__manufacturer__icontains = f_manufacturer)
-    if f_manufacturer_model_name:
-        e = e.filter(general_equipment_module_attributes__manufacturer_model_name__icontains = f_manufacturer_model_name)
-    if f_station_name:
-        e = e.filter(general_equipment_module_attributes__station_name__icontains = f_station_name)
-    if f_accession_number:
-        e = e.filter(accession_number__icontains = f_accession_number)
-    if f_date_after:
-        e = e.filter(study_date__gte = f_date_after)
-    if f_date_before:
-        e = e.filter(study_date__lte = f_date_before)
-    if f_age_min:
-        e = e.filter(patient_study_module_attributes__patient_age_decimal__gte = f_age_min)
-    if f_age_max:
-        e = e.filter(patient_study_module_attributes__patient_age_decimal__lte = f_age_max)
+    tsk.progress = 'Required study filter complete.'
+    tsk.num_records = e.count()
+    tsk.save()
 
-
-    # create a workbook in memory
-    output = StringIO.StringIO()
-    book = Workbook(output, {'default_date_format': 'dd/mm/yyyy',
-                             'strings_to_numbers':  True})
     # Add summary sheet and all data sheet
     summarysheet = book.add_worksheet("Summary")
     wsalldata = book.add_worksheet('All data')       
@@ -141,6 +142,9 @@ def ctxlsx(request):
         ]
         
     # Generate list of protocols in queryset and create worksheets for each
+    tsk.progress = 'Generating list of protocols in the dataset...'
+    tsk.save()
+
     sheetlist = {}
     protocolslist = []
     for exams in e:
@@ -152,6 +156,10 @@ def ctxlsx(request):
             if safeprotocol not in protocolslist:
                 protocolslist.append(safeprotocol)
     protocolslist.sort()
+
+    tsk.progress = 'Creating an Excel safe version of protocol names and creating a worksheet for each...'
+    tsk.save()
+
     for protocol in protocolslist:
         tabtext = protocol.lower().replace(" ","_")
         translation_table = {ord('['):ord('('), ord(']'):ord(')'), ord(':'):ord(';'), ord('*'):ord('#'), ord('?'):ord(';'), ord('/'):ord('|'), ord('\\'):ord('|')}
@@ -178,6 +186,9 @@ def ctxlsx(request):
 
     alldataheaders = commonheaders
 
+    tsk.progress = 'Generating headers for the all data sheet...'
+    tsk.save()
+
     for h in xrange(max_events['ct_radiation_dose__ct_accumulated_dose_data__total_number_of_irradiation_events__max']):
         alldataheaders += [
             'E' + str(h+1) + ' Protocol',
@@ -209,6 +220,10 @@ def ctxlsx(request):
     wsalldata.autofilter(0,0,numrows,numcolumns)
 
     for row,exams in enumerate(e):
+
+        tsk.progress = 'Writing study {0} of {1} to All data sheet and individual protocol sheets'.format(row + 1, numrows)
+        tsk.save()
+
         examdata = [
 			exams.general_equipment_module_attributes_set.get().institution_name,
 			exams.general_equipment_module_attributes_set.get().manufacturer,
@@ -347,8 +362,11 @@ def ctxlsx(request):
     # Could at this point go through each sheet adding on the auto filter as we now know how many of each there are...
     
     # Populate summary sheet
+    tsk.progress = 'Now populating the summary sheet...'
+    tsk.save()
+
     import pkg_resources  # part of setuptools
-    from datetime import datetime
+    import datetime
 
     try:
         vers = pkg_resources.require("openrem")[0].version
@@ -360,7 +378,7 @@ def ctxlsx(request):
     titleformat.set_font_size=(22)
     titleformat.set_font_color=('#FF0000')
     titleformat.set_bold()
-    toplinestring = 'XLSX Export from OpenREM version {0} on {1}'.format(version, str(datetime.now()))
+    toplinestring = 'XLSX Export from OpenREM version {0} on {1}'.format(version, str(datetime.datetime.now()))
     linetwostring = 'OpenREM is copyright 2014 The Royal Marsden NHS Foundation Trust, and available under the GPL. See http://openrem.org'
     summarysheet.write(0,0, toplinestring, titleformat)
     summarysheet.write(1,0, linetwostring)
@@ -400,13 +418,25 @@ def ctxlsx(request):
 
 
     book.close()
-        
+    tsk.progress = 'XLSX book written.'
+    tsk.save()
 
+    xlsxfilename = "ctexport{0}.xlsx".format(datestamp.strftime("%Y%m%d-%H%M%S%f"))
 
+    try:
+        tsk.filename.save(xlsxfilename,File(tmpxlsx))
+    except OSError as e:
+        tsk.progress = "Errot saving export file - please contact an administrator. Error({0}): {1}".format(e.errno, e.strerror)
+        tsk.status = 'ERROR'
+        tsk.save()
+        return
+    except:
+        tsk.progress = "Unexpected error saving export file - please contact an administrator: {0}".format(sys.exc_info()[0])
+        tsk.status = 'ERROR'
+        tsk.save()
+        return
 
-    # construct response
-    output.seek(0)
-    response = HttpResponse(output.read(), mimetype="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
-    response['Content-Disposition'] = "attachment; filename=test.xlsx"
+    tsk.status = 'COMPLETE'
+    tsk.processtime = (datetime.datetime.now() - datestamp).total_seconds()
+    tsk.save()
 
-    return response
