@@ -176,3 +176,49 @@ class MGSummaryListFilter(django_filters.FilterSet):
         elif order_value == '-study_date':
             return ['-study_date','-study_time']
         return super(CTSummaryListFilter, self).get_order_by(order_value)
+
+class DXSummaryListFilter(django_filters.FilterSet):
+    """Filter for DX studies to display in web interface.
+
+    """
+    date_after = django_filters.DateFilter(lookup_type='gte', label='Date from', name='study_date', widget=forms.TextInput(attrs={'class':'datepicker'}))
+    date_before = django_filters.DateFilter(lookup_type='lte', label='Date until', name='study_date', widget=forms.TextInput(attrs={'class':'datepicker'}))
+    study_description = django_filters.CharFilter(lookup_type='icontains', label='Study description')
+    patient_age_min = django_filters.NumberFilter(lookup_type='gt', label='Min age (yrs)', name='patient_study_module_attributes__patient_age_decimal')
+    patient_age_max = django_filters.NumberFilter(lookup_type='lt', label='Max age (yrs)', name='patient_study_module_attributes__patient_age_decimal')
+    institution_name = django_filters.CharFilter(lookup_type='icontains', label='Hospital', name='general_equipment_module_attributes__institution_name')
+    manufacturer = django_filters.CharFilter(lookup_type='icontains', label='Make', name='general_equipment_module_attributes__manufacturer')
+    model_name = django_filters.CharFilter(lookup_type='icontains', label='Model', name='general_equipment_module_attributes__manufacturer_model_name')
+    station_name = django_filters.CharFilter(lookup_type='icontains', label='Station name', name='general_equipment_module_attributes__station_name')
+    accession_number = django_filters.CharFilter(lookup_type='icontains', label='Accession number')
+    class Meta:
+        model = General_study_module_attributes
+        fields = [
+            'date_after', 
+            'date_before', 
+            'institution_name', 
+            'study_description',
+            'patient_age_min',
+            'patient_age_max',
+            'manufacturer', 
+            'model_name',
+            'station_name',
+            'accession_number',
+            ]
+        order_by = (
+            ('-study_date', 'Date of exam (newest first)'),
+            ('study_date', 'Date of exam (oldest first)'),
+            ('general_equipment_module_attributes__institution_name', 'Hospital'),
+            ('general_equipment_module_attributes__manufacturer', 'Make'),
+            ('general_equipment_module_attributes__manufacturer_model_name', 'Model name'),
+            ('general_equipment_module_attributes__station_name', 'Station name'),
+            ('study_description', 'Study description'),
+            ('-projection_xray_radiation_dose__accumulated_xray_dose__accumulated_projection_xray_dose__dose_area_product_total','Total DAP'),
+            )
+
+    def get_order_by(self, order_value):
+        if order_value == 'study_date':
+            return ['study_date', 'study_time']
+        elif order_value == '-study_date':
+            return ['-study_date','-study_time']
+        return super(DXSummaryListFilter, self).get_order_by(order_value)
